#include <zephyr.h>
#include <string.h>
#include <drivers/gpio.h>
#include <hal/nrf_gpio.h>
#include <hal/nrf_gpiote.h>
#include <nrfx_gpiote.h>

#include <nrfx_i2s.h>

#include <drivers/clock_control.h>
#include <drivers/clock_control/nrf_clock_control.h>
#include <logging/log.h>
LOG_MODULE_REGISTER(i2s_tx, LOG_LEVEL_DBG);

//#error Remember to define these pins before you run! // This assume you only have these 3 pins for I2S.

#define I2S_SCK_PIN 	NRF_GPIO_PIN_MAP(0, 11)
#define I2S_LRCK_PIN 	NRF_GPIO_PIN_MAP(0, 12)
#define I2S_MCK_PIN 	NRFX_I2S_PIN_NOT_USED
#define I2S_SDOUT_PIN 	NRF_GPIO_PIN_MAP(1, 8)
#define I2S_SDIN_PIN 	NRF_GPIO_PIN_MAP(1, 6)




#define I2S_DATA_BLOCK_WORDS 512 //How many numbers do we want. time reorded = DATA_BLOCK_WORDS / freq

static uint32_t m_buffer_rx32u[I2S_DATA_BLOCK_WORDS];
static uint32_t tmp[I2S_DATA_BLOCK_WORDS];
nrfx_i2s_buffers_t initial_buffers;

static bool data_ready_flag = false;


ISR_DIRECT_DECLARE(i2s_isr_handler)
{
    data_ready_flag = false;
    nrfx_i2s_irq_handler();
    ISR_DIRECT_PM(); /* PM done after servicing interrupt for best latency
			  */
    return 1;		 /* We should check if scheduling decision should be made */
}

static void data_handler(nrfx_i2s_buffers_t const *p_released, uint32_t status)
{
	//LOG_INF("test I2S0_IRQn:I2S data_handler \n");

    if (NRFX_I2S_STATUS_NEXT_BUFFERS_NEEDED == status)
    {
        nrfx_err_t err = nrfx_i2s_next_buffers_set(&initial_buffers);
        if (err != NRFX_SUCCESS)
        {
            LOG_ERR("Error!, continuing running as if nothing happened, but you should probably investigate.\n");
        }
    }
    if (p_released)
    {
        if (p_released->p_rx_buffer != NULL)
        {
            set_i2s_tx_ready(); //This is used in print_sound()
            coap_client_tx_i2s_mesh();
        }
    }
}

//when i2s recieve ,data_ready_flag will set true ,data will tx to 2.4G
void set_i2s_tx_ready(void)
{
	data_ready_flag = true;
}
void reset_i2s_tx_ready(void)
{
	data_ready_flag = false;
}

uint8_t * get_tx_data_buf_from_i2s(void)
{
	NRFX_ASSERT(initial_buffers.p_rx_buffer != NULL);

	return initial_buffers.p_rx_buffer;
}



void print_sound()
{
    while (!data_ready_flag)
    {
        return;
        //Wait for data. Since we do not want I2S_DATA_BLOCK_WORDS amount of prints inside the interrupt.
    }
    nrfx_i2s_stop();
	
    data_ready_flag = false;

    /** Print the raw data from the I2S microphone */
    /*
    for (int i = 0; i < I2S_DATA_BLOCK_WORDS; i++)
    {
    	printk("%u, ", m_buffer_rx32u[i]); //The audio is automatically saved in m_buffer_rx32u by the interrupt
    	k_sleep(K_MSEC(16));
    }
    printk("\n\n");
    */

    /**  Format the data from Adafruit I2S MEMS Microphone Breakout, then print it	*/

    int64_t sum = 0;
    int64_t words = I2S_DATA_BLOCK_WORDS;

    for (int i = 0; i < I2S_DATA_BLOCK_WORDS; i++)
    {
        memcpy(tmp + i, m_buffer_rx32u + i, sizeof(uint32_t));
        tmp[i] >>= 8;
        sum += tmp[i];
    }

    int64_t mean = sum / words;
    for (int i = 0; i < I2S_DATA_BLOCK_WORDS; i++)
    {
        tmp[i] -= mean;
        LOG_DBG("%d, ", tmp[i]);
        k_sleep(K_MSEC(16));
    }
    LOG_DBG("\n\n");
    /**  End of formatted data print*/
}

int i2s_tx_init(void)
{
	
    memset(&m_buffer_rx32u, 0x00, sizeof(m_buffer_rx32u));
    initial_buffers.p_rx_buffer = m_buffer_rx32u;

    nrfx_i2s_config_t config;

    config.sck_pin        = NRF_GPIO_PIN_MAP(0, 11);
    config.lrck_pin       = NRF_GPIO_PIN_MAP(0, 12);
    config.mck_pin        = NRF_GPIO_PIN_MAP(1, 4);
    config.sdout_pin      = NRF_GPIO_PIN_MAP(1, 8);
    config.sdin_pin       = NRF_GPIO_PIN_MAP(1, 6);
    config.irq_priority   = 1;
    config.mode           = NRF_I2S_MODE_SLAVE;
    config.format         = NRF_I2S_FORMAT_I2S;
    config.alignment      = NRF_I2S_ALIGN_RIGHT;
    config.sample_width   = NRF_I2S_SWIDTH_24BIT_IN32BIT;
    config.channels       = NRF_I2S_CHANNELS_STEREO;
    config.mck_setup      = NRF_I2S_MCK_32MDIV8;
    config.ratio          = NRF_I2S_RATIO_192X;
    config.clksrc         = NRF_I2S_CLKSRC_PCLK32M;
    config.enable_bypass  = false;

    nrfx_err_t err_code = nrfx_i2s_init(&config, data_handler);
    if (err_code != NRFX_SUCCESS)
    {
        LOG_ERR("I2S init error,err_code1 = %d\n", (err_code - NRFX_ERROR_BASE_NUM));
        return (err_code - NRFX_ERROR_BASE_NUM);
    }
    LOG_INF("I2S init success\n");

	
	IRQ_DIRECT_CONNECT(I2S0_IRQn, 0, i2s_isr_handler, 0);
    k_sleep(K_MSEC(200));
	
	
    err_code = nrfx_i2s_start(&initial_buffers, I2S_DATA_BLOCK_WORDS, 0); //start recording
    if (err_code != NRFX_SUCCESS)
    {
        LOG_ERR("I2S start error,err_code1 = %d\n", (err_code - NRFX_ERROR_BASE_NUM));
        return (err_code - NRFX_ERROR_BASE_NUM);
    }
    //nrfx_i2s_stop() //stop recording
    LOG_INF("I2S start success\n");
    return (err_code - NRFX_ERROR_BASE_NUM);
}

